@@ -113,9 +113,7 @@ gboolean        gst_device_has_classes        (GstDevice * device,
                                                const gchar * classes);
 
 
-#ifdef G_DEFINE_AUTOPTR_CLEANUP_FUNC
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstDevice, gst_object_unref)
-#endif
 
 G_END_DECLS
 
